/*
 *  objectTracker.cpp
 *  opencv object tracking
 *
 *  Created by Ryan McGee and Tim Wood on 3/7/09.
 *
 */

#include "objectTracker.h"

ObjectTracker::ObjectTracker(){ //consructor
	
	nHistBins = 30;           // number of histogram bins
	histRange[0] = 0;
	histRange[1] = 180;

	// saturation and value ranges
	valMin = 60;
	valMax = 255;
	satMin = 70;     
	
	hsvImg = 0; // the input image converted to HSV color mode
	hueImg = 0; // the Hue channel of the HSV image
	mask = 0; // bit mask for pixels within designated range of s and v
	probImg = 0; // the object probability estimates for each pixel
	hist = 0; // histogram of hue in the original image
	
}

ObjectTracker::~ObjectTracker(){ //destructor

	cvReleaseImage( &hsvImg );
	cvReleaseImage( &hueImg );
	cvReleaseImage( &mask );
	cvReleaseImage( &probImg );
	
	cvReleaseHist( &hist );
}

void ObjectTracker::init(IplImage* image, CvRect inputRect){

	float *p = histRange;

	hsvImg = cvCreateImage( cvGetSize(image), IPL_DEPTH_8U, 3 );
	hueImg = cvCreateImage( cvGetSize(image), IPL_DEPTH_8U, 1 );
	mask = cvCreateImage( cvGetSize(image), IPL_DEPTH_8U, 1 );
	probImg = cvCreateImage( cvGetSize(image), IPL_DEPTH_8U, 1 );
	
	hist = cvCreateHist( 1, &nHistBins, CV_HIST_ARRAY, &p, 1 ); 
	
	convertToHSV(image);
	

	float maxHistVal;

    cvSetImageROI( hueImg, inputRect );
    cvSetImageROI( mask,   inputRect );

	//calc histogram of masked image(includes only pixels within sat and val range)
    cvCalcHist( &hueImg, hist, 0, mask );	
    cvGetMinMaxHistValue( hist, 0, &maxHistVal, 0, 0 );		//get max of bins
    if(maxHistVal)cvConvertScale( hist->bins, hist->bins, 255.0/maxHistVal, 0 );	//normalize histogram values
	else cvConvertScale( hist->bins, hist->bins, 0, 0);

    cvResetImageROI( hueImg );
    cvResetImageROI( mask );
	
	prevTrackRect = inputRect;

}

CvBox2D ObjectTracker::track(IplImage* image){

	CvConnectedComp components;
	
	convertToHSV(image);
	
	cvCalcBackProject( &hueImg, probImg, hist );
	cvAnd( probImg, mask, probImg, 0 );
	
	cvCamShift( probImg, prevTrackRect, cvTermCriteria( CV_TERMCRIT_EPS | CV_TERMCRIT_ITER, 20, 0.5 ), &components, &trackBox );
	
	//keep track of rectangle for next iteration
	prevTrackRect = components.rect;
	trackBox.angle = -trackBox.angle;

	return trackBox;

}

void ObjectTracker::convertToHSV(IplImage* image){

	cvCvtColor(image, hsvImg, CV_BGR2HSV );
	
	//set mask to include only pixels within ranges...
	cvInRangeS( hsvImg, cvScalar(0, satMin, valMin, 0), cvScalar(180, 256, valMax, 0), mask );

	cvSplit( hsvImg, hueImg, 0, 0, 0 );

}